==> spirit_hub-0.0.1.6.tar.gz: spirit_hub-0.0.1.6/spec/models/tagsets_spec.rb <==
# frozen_string_literal: true

require 'rails_helper'

RSpec.describe(Tagset, type: :model) do
  before do
    SpiritHub::HubdocCycle.import_cycle_from_json
  end

  describe 'update_operation_title' do
    let!(:operation) { create(:operation, title: 'Operation test') }

    it 'updates operation title' do
      tagset = create(:tagset, operation:)
      expect(tagset.operation_title).to eq('Operation test')

      operation.update(title: 'Operation test 2')
      tagset.reload

      expect(tagset.operation_title).to eq('Operation test 2')
    end
  end

  describe 'update_informed_fields' do
    let!(:user1) { create(:user, first_name: 'User', last_name: '1') }
    let!(:user2) { create(:user, first_name: 'User', last_name: '2') }
    let!(:user3) { create(:user, first_name: 'User', last_name: '3') }

    let!(:tagset) { create(:tagset) }

    it 'default' do
      expect(tagset.informed).to eq([])
      expect(tagset.informed_main).to eq([])
      expect(tagset.informed_cc).to eq([])
      expect(tagset.informed_bcc).to eq([])
    end

    it 'add user' do
      tagset.update!(informed_main: [user1.id], informed_cc: [user2.id], informed_bcc: [user3.id])
      expect(tagset.reload.informed_main).to eq([user1.id])
      expect(tagset.informed_cc).to eq([user2.id])
      expect(tagset.informed_bcc).to eq([user3.id])
      expect(tagset.informed).to eq([user1.id, user2.id, user3.id])
    end

    it 'add users and remove them' do
      tagset.update(informed_main: [user1.id, user2.id], informed_bcc: [user3.id])
      expect(tagset.reload.informed_main).to eq([user1.id, user2.id])
      expect(tagset.informed_cc).to eq([])
      expect(tagset.informed_bcc).to eq([user3.id])
      expect(tagset.informed).to eq([user1.id, user2.id, user3.id])

      tagset.update(informed_bcc: [])
      expect(tagset.reload.informed_main).to eq([user1.id, user2.id])
      expect(tagset.informed_cc).to eq([])
      expect(tagset.informed_bcc).to eq([])
      expect(tagset.informed).to eq([user1.id, user2.id])

      tagset.update(informed: [user3.id])
      expect(tagset.reload.informed_main).to eq([user3.id])
      expect(tagset.informed_cc).to eq([])
      expect(tagset.informed_bcc).to eq([])
      expect(tagset.informed).to eq([user3.id])
    end
  end

  describe 'add_hubdoc_label' do
    let!(:tagset) { create(:tagset) }

    it 'default' do
      expect(tagset.reload.labels).to include('hubdoc')
    end

    it 'dont add label if already present' do
      tagset.update(labels: %w[hubdoc other])
      expect(tagset.reload.labels).to eq(%w[hubdoc other])
    end
  end

  describe 'update_children_parent_ids' do
    let!(:tagset) { create(:tagset) }
    let!(:child1) { create(:tagset) }
    let!(:child2) { create(:tagset) }

    it 'default' do
      expect(tagset.reload.parent_tagset_ids).to eq([])
      expect(child1.reload.parent_tagset_ids).to eq([])
      expect(child2.reload.parent_tagset_ids).to eq([])
    end

    it 'add child' do
      tagset.update(tagset_ids: [child1.id.to_s])
      expect(tagset.reload.parent_tagset_ids).to eq([])
      expect(child1.reload.parent_tagset_ids).to eq([tagset.id])
    end

    it 'add 2 children and remove one' do
      tagset.update(tagset_ids: [child1.id.to_s, child2.id.to_s])
      expect(tagset.reload.parent_tagset_ids).to eq([])
      expect(child1.reload.parent_tagset_ids).to eq([tagset.id])
      expect(child2.reload.parent_tagset_ids).to eq([tagset.id])

      tagset.update(tagset_ids: [child1.id.to_s])
      expect(tagset.reload.parent_tagset_ids).to eq([])
      expect(child1.reload.parent_tagset_ids).to eq([tagset.id])
      expect(child2.reload.parent_tagset_ids).to eq([])
    end
  end

  describe 'update_completeness' do
    let!(:user) { create(:user) }
    let!(:tagset1) { create(:tagset) }
    let!(:tagset2) { create(:tagset) }
    let!(:completeness) { create(:completeness, tagset_id: tagset2.id, object_ids: [tagset1.id]) }

    it 'default' do
      expect(completeness.reload.object_ids).to eq([tagset1.id])
    end

    it 'when destroy tagset' do
      tagset1.current_user_id = user.id
      tagset1.destroy
      expect(completeness.reload.object_ids).to eq([])
    end
  end

  describe 'share_to_users' do
    subject do
      tagset.share_to_users(
        to: [user1.id.to_s],
        cc: [user2.id.to_s],
        bcc: [user3.id.to_s],
        subject: message_subject,
        message:,
        current_user: user4
      )
    end

    let(:tagset) { create(:tagset_doc, :with_file) }
    let(:user1) { create(:user) }
    let(:user2) { create(:user) }
    let(:user3) { create(:user) }
    let(:user4) { create(:user) }

    let(:message_subject) { Faker::Lorem.sentence }
    let(:message) { Faker::Lorem.sentence }

    it 'shares the doc to the user using the underlying service obj' do
      expect(SpiritHub::TagsetShare).to receive(:call).with(
        tagset:,
        to: [user1.id.to_s],
        cc: [user2.id.to_s],
        bcc: [user3.id.to_s],
        subject: message_subject,
        message:,
        current_user: user4
      )

      subject
    end
  end

  describe 'update_docs_fields' do
    let!(:tagset) { create(:tagset) }

    it 'without docs' do
      expect(tagset.reload.file_ext).to be_nil
      expect(tagset.reload.doc_file_size).to eq(nil)
    end

    it 'with one doc' do
      doc = create(:doc, file_ext: 'jpeg', file_size: 1000)
      tagset.update(doc_ids: [doc.id])
      expect(tagset.reload.file_ext).to eq('jpeg')
      expect(tagset.reload.doc_file_size).to eq(1000)
    end

    it 'with multiple docs' do
      doc1 = create(:doc, file_ext: 'jpeg', file_size: 1000)
      doc2 = create(:doc, file_ext: 'pdf', file_size: 2000)
      tagset.update(doc_ids: [doc1.id, doc2.id])
      expect(tagset.reload.file_ext).to eq('multiple')
      expect(tagset.reload.doc_file_size).to eq(3000)
    end
  end

  describe 'purge!' do
    subject { tagset.purge! }

    let(:tagset) { create(:tagset_doc) }
    let(:user) { create(:user) }

    before do
      tagset.current_user_id = user.id
    end

    it 'destroys the record' do
      expect(tagset).to be_persisted
      subject
      expect(tagset).not_to be_persisted
      expect(Tagset.find(tagset.id)).to be_nil
    end

    context 'when the tagset is a child of another tagset' do
      let!(:tagset_parent1) { create(:tagset_directory, tagset_ids: ['other_tagset_id', tagset.id.to_s]) }
      let!(:tagset_parent2) { create(:tagset_directory, tagset_ids: [tagset.id.to_s]) }

      it 'clears the references from the parents' do
        expect(tagset_parent1.tagset_ids).to include(tagset.id.to_s)
        expect(tagset_parent2.tagset_ids).to include(tagset.id.to_s)

        subject

        expect(tagset_parent1.reload.tagset_ids).to eq ['other_tagset_id']
        expect(tagset_parent2.reload.tagset_ids).to eq []
      end
    end

    context 'when the tagset is a associated to a completeness rule' do
      let!(:completeness1) { create(:completeness, tagset: tagset_owner, object_ids: ['other_tagset_id', tagset.id]) }
      let!(:completeness2) { create(:completeness, tagset: tagset_owner, object_ids: [tagset.id]) }
      let(:tagset_owner) { create(:tagset_directory) }

      it 'clears the references from the parents' do
        expect(completeness1.object_ids).to include(tagset.id)
        expect(completeness2.object_ids).to include(tagset.id)

        subject

        expect(completeness1.reload.object_ids).to eq ['other_tagset_id']
        expect(completeness2.reload.object_ids).to eq []
      end
    end
  end

  describe 'create_tagset_doc_from_file' do
    let(:user) { create(:user) }
    let(:file1) { double('file1', original_filename: 'document1.pdf', read: 'File content 1') }
    let(:file2) { double('file2', original_filename: 'document2.pdf', read: 'File content 2') }

    context 'with multiple files' do
      it 'creates docs and a tagset with correct attributes' do
        files = [file1, file2]
        tagset = Tagset.create_tagset_doc_from_file(files, user)

        expect(tagset).to be_persisted
        expect(tagset.responsible).to eq(user.id)
        expect(tagset.type).to eq('document')
        expect(tagset.title).to eq(file1.original_filename)
        expect(tagset.doc_ids.size).to eq(files.size)

        tagset.doc_ids.each_with_index do |doc_id, index|
          doc = Doc.find(doc_id)
          file = files[index]

          expect(doc.responsible).to eq(user.id)
          expect(doc.title).to eq(file.original_filename)
          expect(doc.type).to eq('file')
        end
      end
    end

    context 'with a single file' do
      it 'creates a doc and a tagset with correct attributes' do
        files = file1

        tagset = Tagset.create_tagset_doc_from_file(files, user)

        expect(tagset.responsible).to eq(user.id)
        expect(tagset.type).to eq('document')
        expect(tagset.title).to eq(file1.original_filename)
        expect(tagset.doc_ids.size).to eq(1)

        doc = Doc.find(tagset.doc_ids.first)

        expect(doc.responsible).to eq(user.id)
        expect(doc.title).to eq(file1.original_filename)
        expect(doc.type).to eq('file')
      end
    end
  end

  describe 'add_tagset_doc_to_object' do
    let!(:user) { create(:user) }
    let!(:model) { create(:tagset_doc) }
    let!(:operation) { create(:operation, title: 'Operation 1') }
    let(:tagset) { create(:tagset, operation_id: operation.id) }
    let(:completeness) { create(:completeness, title: 'Completness', tagset_id: tagset.id) }

    before do
      SpiritHub::HubdocCycle.import_cycle_from_json
    end

    context 'when a Tagset is found by object_id' do
      it 'add the model to the tagset' do
        model.add_tagset_doc_to_object(tagset.id, user)

        classify_state = State.find_by(key: :classify)
        expect(model.reload.state_id).to eq(classify_state.id)
        expect(model.current_state).to eq('Classé')
        expect(model.operation_id).to eq(tagset.operation_id)
        expect(model.operation_title).to eq(tagset.operation_title)
        expect(tagset.reload.tagset_ids).to include(model.id)
      end
    end

    context 'when a Completeness is found by object_id' do
      it 'add the model to the completeness' do
        model.add_tagset_doc_to_object(completeness.id, user)

        classify_state = State.find_by(key: :classify)
        expect(model.reload.state_id).to eq(classify_state.id)
        expect(model.current_state).to eq('Classé')
        expect(model.title).to eq(completeness.title)
        expect(model.operation_id).to eq(tagset.operation_id)
        expect(model.operation_title).to eq(tagset.operation_title)
        expect(completeness.reload.object_ids).to include(model.id)
        expect(tagset.reload.tagset_ids).to include(model.id.to_s)
      end
    end

    context 'when has a completeness and a tagset' do
      let!(:other_tagset) { create(:tagset, title: 'Other Tagset', operation_id: operation.id) }
      let!(:other_completeness) { create(:completeness, title: 'Other Completness', tagset_id: other_tagset.id) }

      before do
        model.add_tagset_doc_to_object(other_completeness.id, user)
      end

      it 'remove the previous completeness and tagset' do
        expect(other_completeness.reload.object_ids.to_a).to include(model.id)
        expect(other_tagset.reload.tagset_ids.to_a).to include(model.id.to_s)

        model.add_tagset_doc_to_object(completeness.id, user)

        expect(completeness.reload.object_ids.to_a).to include(model.id)
        expect(tagset.reload.tagset_ids.to_a).to include(model.id.to_s)
        expect(other_completeness.reload.object_ids.to_a).not_to include(model.id)
        expect(other_tagset.reload.tagset_ids.to_a).not_to include(model.id.to_s)
      end
    end
  end

  describe 'add_doc_to_tagset_doc' do
    let!(:user) { create(:user) }
    let!(:tagset_doc) { create(:tagset_doc) }
    let(:file1) { double('file1', original_filename: 'document1.pdf', read: 'File content 1') }
    let(:file2) { double('file2', original_filename: 'document2.pdf', read: 'File content 2') }

    it 'with multiple files' do
      files = [file1, file2]
      tagset_doc.add_doc_to_tagset_doc(files, user)

      expect(tagset_doc.doc_ids.size).to eq(files.size)

      tagset_doc.doc_ids.each_with_index do |doc_id, index|
        doc = Doc.find(doc_id)
        file = files[index]

        expect(doc.responsible).to eq(user.id)
        expect(doc.title).to eq(file.original_filename)
        expect(doc.type).to eq('file')
      end
    end

    it 'with a single file' do
      tagset_doc.add_doc_to_tagset_doc(file1, user)
      expect(tagset_doc.doc_ids.size).to eq(1)

      doc = Doc.find(tagset_doc.doc_ids.first)

      expect(doc.responsible).to eq(user.id)
      expect(doc.title).to eq(file1.original_filename)
      expect(doc.type).to eq('file')
    end
  end

  describe 'remove_doc_from_tagset_doc' do
    let!(:user) { create(:user) }
    let!(:tagset_doc) { create(:tagset_doc, :with_file) }

    it 'works' do
      doc = Doc.find(tagset_doc.doc_ids.first)
      tagset_doc.remove_doc_from_tagset_doc(doc, user)

      expect(tagset_doc.doc_ids).to eq([])
      expect(Doc.find_by(id: doc.id)).to be_nil
    end
  end
end

==> spirit_hub-0.0.1.7.tar.gz: spirit_hub-0.0.1.7/spec/models/tagsets_spec.rb <==
# frozen_string_literal: true

require 'rails_helper'

RSpec.describe(Tagset, type: :model) do
  before do
    SpiritHub::HubdocCycle.import_cycle_from_json
  end

  describe 'update_operation_title' do
    let!(:operation) { create(:operation, title: 'Operation test') }

    it 'updates operation title' do
      tagset = create(:tagset, operation:)
      expect(tagset.operation_title).to eq('Operation test')

      operation.update(title: 'Operation test 2')
      tagset.reload

      expect(tagset.operation_title).to eq('Operation test 2')
    end
  end

  describe 'update_informed_fields' do
    let!(:user1) { create(:user, first_name: 'User', last_name: '1') }
    let!(:user2) { create(:user, first_name: 'User', last_name: '2') }
    let!(:user3) { create(:user, first_name: 'User', last_name: '3') }

    let!(:tagset) { create(:tagset) }

    it 'default' do
      expect(tagset.informed).to eq([])
      expect(tagset.informed_main).to eq([])
      expect(tagset.informed_cc).to eq([])
      expect(tagset.informed_bcc).to eq([])
    end

    it 'add user' do
      tagset.update!(informed_main: [user1.id], informed_cc: [user2.id], informed_bcc: [user3.id])
      expect(tagset.reload.informed_main).to eq([user1.id])
      expect(tagset.informed_cc).to eq([user2.id])
      expect(tagset.informed_bcc).to eq([user3.id])
      expect(tagset.informed).to eq([user1.id, user2.id, user3.id])
    end

    it 'add users and remove them' do
      tagset.update(informed_main: [user1.id, user2.id], informed_bcc: [user3.id])
      expect(tagset.reload.informed_main).to eq([user1.id, user2.id])
      expect(tagset.informed_cc).to eq([])
      expect(tagset.informed_bcc).to eq([user3.id])
      expect(tagset.informed).to eq([user1.id, user2.id, user3.id])

      tagset.update(informed_bcc: [])
      expect(tagset.reload.informed_main).to eq([user1.id, user2.id])
      expect(tagset.informed_cc).to eq([])
      expect(tagset.informed_bcc).to eq([])
      expect(tagset.informed).to eq([user1.id, user2.id])

      tagset.update(informed: [user3.id])
      expect(tagset.reload.informed_main).to eq([user3.id])
      expect(tagset.informed_cc).to eq([])
      expect(tagset.informed_bcc).to eq([])
      expect(tagset.informed).to eq([user3.id])
    end
  end

  describe 'add_hubdoc_label' do
    let!(:tagset) { create(:tagset) }

    it 'default' do
      expect(tagset.reload.labels).to include('hubdoc')
    end

    it 'dont add label if already present' do
      tagset.update(labels: %w[hubdoc other])
      expect(tagset.reload.labels).to eq(%w[hubdoc other])
    end
  end

  describe 'update_children_parent_ids' do
    let!(:tagset) { create(:tagset) }
    let!(:child1) { create(:tagset) }
    let!(:child2) { create(:tagset) }

    it 'default' do
      expect(tagset.reload.parent_tagset_ids).to eq([])
      expect(child1.reload.parent_tagset_ids).to eq([])
      expect(child2.reload.parent_tagset_ids).to eq([])
    end

    it 'add child' do
      tagset.update(tagset_ids: [child1.id.to_s])
      expect(tagset.reload.parent_tagset_ids).to eq([])
      expect(child1.reload.parent_tagset_ids).to eq([tagset.id])
    end

    it 'add 2 children and remove one' do
      tagset.update(tagset_ids: [child1.id.to_s, child2.id.to_s])
      expect(tagset.reload.parent_tagset_ids).to eq([])
      expect(child1.reload.parent_tagset_ids).to eq([tagset.id])
      expect(child2.reload.parent_tagset_ids).to eq([tagset.id])

      tagset.update(tagset_ids: [child1.id.to_s])
      expect(tagset.reload.parent_tagset_ids).to eq([])
      expect(child1.reload.parent_tagset_ids).to eq([tagset.id])
      expect(child2.reload.parent_tagset_ids).to eq([])
    end
  end

  describe 'update_completeness' do
    let!(:user) { create(:user) }
    let!(:tagset1) { create(:tagset) }
    let!(:tagset2) { create(:tagset) }
    let!(:completeness) { create(:completeness, tagset_id: tagset2.id, object_ids: [tagset1.id]) }

    it 'default' do
      expect(completeness.reload.object_ids).to eq([tagset1.id])
    end

    it 'when destroy tagset' do
      tagset1.current_user_id = user.id
      tagset1.destroy
      expect(completeness.reload.object_ids).to eq([])
    end
  end

  describe 'share_to_users' do
    subject do
      tagset.share_to_users(
        to: [user1.id.to_s],
        cc: [user2.id.to_s],
        bcc: [user3.id.to_s],
        subject: message_subject,
        message:,
        current_user: user4
      )
    end

    let(:tagset) { create(:tagset_doc, :with_file, title: 'Super Doc') }
    let(:user1) { create(:user, email: 'email1@test.com') }
    let(:user2) { create(:user, email: 'email2@test.com') }
    let(:user3) { create(:user, email: 'email3@test.com') }
    let(:user4) { create(:user, first_name: 'John', last_name: 'Doe') }

    let(:message_subject) { Faker::Lorem.sentence }
    let(:message) { Faker::Lorem.sentence }

    it 'shares the doc to the user using the underlying service obj' do
      expect(SpiritHub::TagsetShare).to receive(:call).with(
        tagset:,
        to: [user1.id.to_s],
        cc: [user2.id.to_s],
        bcc: [user3.id.to_s],
        subject: message_subject,
        message:,
        current_user: user4
      )

      subject
    end
  end

  describe 'update_docs_fields' do
    let!(:tagset) { create(:tagset) }

    it 'without docs' do
      expect(tagset.reload.file_ext).to be_nil
      expect(tagset.reload.doc_file_size).to eq(nil)
    end

    it 'with one doc' do
      doc = create(:doc, file_ext: 'jpeg', file_size: 1000)
      tagset.update(doc_ids: [doc.id])
      expect(tagset.reload.file_ext).to eq('jpeg')
      expect(tagset.reload.doc_file_size).to eq(1000)
    end

    it 'with multiple docs' do
      doc1 = create(:doc, file_ext: 'jpeg', file_size: 1000)
      doc2 = create(:doc, file_ext: 'pdf', file_size: 2000)
      tagset.update(doc_ids: [doc1.id, doc2.id])
      expect(tagset.reload.file_ext).to eq('multiple')
      expect(tagset.reload.doc_file_size).to eq(3000)
    end
  end

  describe 'purge!' do
    subject { tagset.purge! }

    let(:tagset) { create(:tagset_doc) }
    let(:user) { create(:user) }

    before do
      tagset.current_user_id = user.id
    end

    it 'destroys the record' do
      expect(tagset).to be_persisted
      subject
      expect(tagset).not_to be_persisted
      expect(Tagset.find(tagset.id)).to be_nil
    end

    context 'when the tagset is a child of another tagset' do
      let!(:tagset_parent1) { create(:tagset_directory, tagset_ids: ['other_tagset_id', tagset.id.to_s]) }
      let!(:tagset_parent2) { create(:tagset_directory, tagset_ids: [tagset.id.to_s]) }

      it 'clears the references from the parents' do
        expect(tagset_parent1.tagset_ids).to include(tagset.id.to_s)
        expect(tagset_parent2.tagset_ids).to include(tagset.id.to_s)

        subject

        expect(tagset_parent1.reload.tagset_ids).to eq ['other_tagset_id']
        expect(tagset_parent2.reload.tagset_ids).to eq []
      end
    end

    context 'when the tagset is a associated to a completeness rule' do
      let!(:completeness1) { create(:completeness, tagset: tagset_owner, object_ids: ['other_tagset_id', tagset.id]) }
      let!(:completeness2) { create(:completeness, tagset: tagset_owner, object_ids: [tagset.id]) }
      let(:tagset_owner) { create(:tagset_directory) }

      it 'clears the references from the parents' do
        expect(completeness1.object_ids).to include(tagset.id)
        expect(completeness2.object_ids).to include(tagset.id)

        subject

        expect(completeness1.reload.object_ids).to eq ['other_tagset_id']
        expect(completeness2.reload.object_ids).to eq []
      end
    end
  end

  describe 'create_tagset_doc_from_file' do
    let(:user) { create(:user) }
    let(:file1) { double('file1', original_filename: 'document1.pdf', read: 'File content 1') }
    let(:file2) { double('file2', original_filename: 'document2.pdf', read: 'File content 2') }

    context 'with multiple files' do
      it 'creates docs and a tagset with correct attributes' do
        files = [file1, file2]
        tagset = Tagset.create_tagset_doc_from_file(files, user)

        expect(tagset).to be_persisted
        expect(tagset.responsible).to eq(user.id)
        expect(tagset.type).to eq('document')
        expect(tagset.title).to eq(file1.original_filename)
        expect(tagset.doc_ids.size).to eq(files.size)

        tagset.doc_ids.each_with_index do |doc_id, index|
          doc = Doc.find(doc_id)
          file = files[index]

          expect(doc.responsible).to eq(user.id)
          expect(doc.title).to eq(file.original_filename)
          expect(doc.type).to eq('file')
        end
      end
    end

    context 'with a single file' do
      it 'creates a doc and a tagset with correct attributes' do
        files = file1

        tagset = Tagset.create_tagset_doc_from_file(files, user)

        expect(tagset.responsible).to eq(user.id)
        expect(tagset.type).to eq('document')
        expect(tagset.title).to eq(file1.original_filename)
        expect(tagset.doc_ids.size).to eq(1)

        doc = Doc.find(tagset.doc_ids.first)

        expect(doc.responsible).to eq(user.id)
        expect(doc.title).to eq(file1.original_filename)
        expect(doc.type).to eq('file')
      end
    end
  end

  describe 'add_tagset_doc_to_object' do
    let!(:user) { create(:user) }
    let!(:model) { create(:tagset_doc) }
    let!(:operation) { create(:operation, title: 'Operation 1') }
    let(:tagset) { create(:tagset, operation_id: operation.id) }
    let(:completeness) { create(:completeness, title: 'Completness', tagset_id: tagset.id) }

    context 'when a Tagset is found by object_id' do
      it 'add the model to the tagset' do
        model.add_tagset_doc_to_object(tagset.id, user)

        classify_state = State.find_by(key: :classify)
        expect(model.reload.state_id).to eq(classify_state.id)
        expect(model.current_state).to eq('Classé')
        expect(model.operation_id).to eq(tagset.operation_id)
        expect(model.operation_title).to eq(tagset.operation_title)
        expect(tagset.reload.tagset_ids).to include(model.id)
      end
    end

    context 'when a Completeness is found by object_id' do
      it 'add the model to the completeness' do
        model.add_tagset_doc_to_object(completeness.id, user)

        classify_state = State.find_by(key: :classify)
        expect(model.reload.state_id).to eq(classify_state.id)
        expect(model.current_state).to eq('Classé')
        expect(model.title).to eq(completeness.title)
        expect(model.operation_id).to eq(tagset.operation_id)
        expect(model.operation_title).to eq(tagset.operation_title)
        expect(completeness.reload.object_ids).to include(model.id)
        expect(tagset.reload.tagset_ids).to include(model.id.to_s)
      end
    end

    context 'when has a completeness and a tagset' do
      let!(:other_tagset) { create(:tagset, title: 'Other Tagset', operation_id: operation.id) }
      let!(:other_completeness) { create(:completeness, title: 'Other Completness', tagset_id: other_tagset.id) }

      before do
        model.add_tagset_doc_to_object(other_completeness.id, user)
      end

      it 'remove the previous completeness and tagset' do
        expect(other_completeness.reload.object_ids.to_a).to include(model.id)
        expect(other_tagset.reload.tagset_ids.to_a).to include(model.id.to_s)

        model.add_tagset_doc_to_object(completeness.id, user)

        expect(completeness.reload.object_ids.to_a).to include(model.id)
        expect(tagset.reload.tagset_ids.to_a).to include(model.id.to_s)
        expect(other_completeness.reload.object_ids.to_a).not_to include(model.id)
        expect(other_tagset.reload.tagset_ids.to_a).not_to include(model.id.to_s)
      end
    end
  end

  describe 'add_doc_to_tagset_doc' do
    let!(:user) { create(:user) }
    let!(:tagset_doc) { create(:tagset_doc) }
    let(:file1) { double('file1', original_filename: 'document1.pdf', read: 'File content 1') }
    let(:file2) { double('file2', original_filename: 'document2.pdf', read: 'File content 2') }

    it 'with multiple files' do
      files = [file1, file2]
      tagset_doc.add_doc_to_tagset_doc(files, user)

      expect(tagset_doc.doc_ids.size).to eq(files.size)

      tagset_doc.doc_ids.each_with_index do |doc_id, index|
        doc = Doc.find(doc_id)
        file = files[index]

        expect(doc.responsible).to eq(user.id)
        expect(doc.title).to eq(file.original_filename)
        expect(doc.type).to eq('file')
      end
    end

    it 'with a single file' do
      tagset_doc.add_doc_to_tagset_doc(file1, user)
      expect(tagset_doc.doc_ids.size).to eq(1)

      doc = Doc.find(tagset_doc.doc_ids.first)

      expect(doc.responsible).to eq(user.id)
      expect(doc.title).to eq(file1.original_filename)
      expect(doc.type).to eq('file')
    end
  end

  describe 'remove_doc_from_tagset_doc' do
    let!(:user) { create(:user) }
    let!(:tagset_doc) { create(:tagset_doc, :with_file) }

    it 'works' do
      doc = Doc.find(tagset_doc.doc_ids.first)
      tagset_doc.remove_doc_from_tagset_doc(doc, user)

      expect(tagset_doc.doc_ids).to eq([])
      expect(Doc.find_by(id: doc.id)).to be_nil
    end
  end

  describe 'process_to_users' do
    let!(:user1) { create(:user) }
    let!(:user2) { create(:user) }
    let!(:tagset_doc) { create(:tagset_doc) }

    before do
      SpiritHub::HubdocCycle.change_state(tagset_doc, :to_validate, user1)
      tagset_doc.save
    end

    before { tagset_doc.reload.process_to_users(state_key, current_user: user1, recipients: [user2.id], message: 'lol') }

    context 'when the state is validate' do
      let!(:state_key) { :validate }

      it 'changes the state to validate and give it to other user' do
        subject
        LogJob.new.perform

        expect(tagset_doc.reload.state.key).to eq('validate')
        expect(tagset_doc.reload.current_state).to eq('Validé')
        expect(tagset_doc.reload.accounted).to eq([user2.id])
        expect(tagset_doc.logs.first.code).to eq('DOC_VALIDATED')
      end
    end

    context 'when the state is refuse' do
      let!(:state_key) { :refuse }

      it 'changes the state to refuse and give it to other user' do
        subject
        LogJob.new.perform

        expect(tagset_doc.reload.state.key).to eq('refuse')
        expect(tagset_doc.reload.current_state).to eq('Refusé')
        expect(tagset_doc.reload.accounted).to eq([user2.id])
        expect(tagset_doc.logs.first.code).to eq('DOC_REFUSED')
      end
    end
  end
end
